@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2009 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2010 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Portions Copyright (c) 1997 Alan Citterman. All rights reserved.
 #
@@ -30,7 +30,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.69";
+$VERSION = "0.71";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -59,6 +59,7 @@ my %def_attr = (
     sep_char		=> ',',
     eol			=> '',
     always_quote	=> 0,
+    quote_space		=> 1,
     binary		=> 0,
     keep_meta_info	=> 0,
     allow_loose_quotes	=> 0,
@@ -82,6 +83,20 @@ my %def_attr = (
     );
 my $last_new_err = Text::CSV_XS->SetDiag (0);
 
+sub _check_sanity
+{
+    my $attr = shift;
+    for (qw( sep_char quote_char escape_char )) {
+	exists $attr->{$_} && defined $attr->{$_} && $attr->{$_} =~ m/[\r\n]/ and
+	    return 1003;
+	}
+    $attr->{allow_whitespace} and
+      (defined $attr->{quote_char}  && $attr->{quote_char}  =~ m/^[ \t]$/) ||
+      (defined $attr->{escape_char} && $attr->{escape_char} =~ m/^[ \t]$/) and
+	return 1002;
+    return 0;
+    } # _check_sanity
+
 sub new
 {
     $last_new_err = SetDiag (undef, 1000,
@@ -99,14 +114,14 @@ sub new
 	    }
 #	croak?
 	$last_new_err = SetDiag (undef, 1000, "INI - Unknown attribute '$_'");
+	$attr->{auto_diag} and error_diag ();
 	return;
 	}
 
     my $self  = {%def_attr, %{$attr}};
-    if ($self->{allow_whitespace} and
-	    (defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/) ||
-	    (defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/)) {
-	$last_new_err = SetDiag (undef, 1002);
+    if (my $ec = _check_sanity ($self)) {
+	$last_new_err = SetDiag (undef, $ec);
+	$attr->{auto_diag} and error_diag ();
 	return;
 	}
 
@@ -134,16 +149,19 @@ my %_cache_id = ( # Only expose what is accessed from within PM
     verbatim		=> 22,
     empty_is_undef	=> 23,
     auto_diag		=> 24,
-    _is_bound		=> 25,	# 25 .. 28
+    quote_space		=> 25,
+    _is_bound		=> 26,	# 26 .. 29
     );
 
 # A `character'
 sub _set_attr_C
 {
-    my ($self, $name, $val) = @_;
+    my ($self, $name, $val, $ec) = @_;
     defined $val or $val = 0;
     $] >= 5.008002 and utf8::decode ($val);
     $self->{$name} = $val;
+    $ec = _check_sanity ($self) and
+	croak ($self->SetDiag ($ec));
     $self->_cache_set ($_cache_id{$name}, $val);
     } # _set_attr_C
 
@@ -171,8 +189,6 @@ sub quote_char
     my $self = shift;
     if (@_) {
 	my $qc = shift;
-	defined $qc && $qc =~ m/^[ \t]$/ && $self->{allow_whitespace} and
-	    croak ($self->SetDiag (1002));
 	$self->_set_attr_C ("quote_char", $qc);
 	}
     $self->{quote_char};
@@ -183,8 +199,6 @@ sub escape_char
     my $self = shift;
     if (@_) {
 	my $ec = shift;
-	defined $ec && $ec =~ m/^[ \t]$/ && $self->{allow_whitespace} and
-	    croak ($self->SetDiag (1002));
 	$self->_set_attr_C ("escape_char", $ec);
 	}
     $self->{escape_char};
@@ -216,6 +230,13 @@ sub always_quote
     $self->{always_quote};
     } # always_quote
 
+sub quote_space
+{
+    my $self = shift;
+    @_ and $self->_set_attr_X ("quote_space", shift);
+    $self->{quote_space};
+    } # quote_space
+
 sub binary
 {
     my $self = shift;
@@ -332,24 +353,29 @@ sub error_diag
 
     my $context = wantarray;
     unless (defined $context) {	# Void context, auto-diag
-	if ($diag[0] && $diag[0] != 2012 && $self && ref $self) {
+	if ($diag[0] && $diag[0] != 2012) {
 	    my $msg = "# CSV_XS ERROR: $diag[0] - $diag[1]\n";
+	    if ($self && ref $self) {	# auto_diag
 
-	    my $lvl = $self->{auto_diag};
-	    if ($lvl < 2) {
-		my @c = caller (2);
-		if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
-		    my $hints = $c[10];
-		    (exists $hints->{autodie} && $hints->{autodie} or
-		     exists $hints->{"guard Fatal"} &&
-		    !exists $hints->{"no Fatal"}) and
-			$lvl++;
-		    # Future releases of autodie will probably set $^H{autodie}
-		    #  to "autodie @args", like "autodie :all" or "autodie open"
-		    #  so we can/should check for "open" or "new"
+		my $lvl = $self->{auto_diag};
+		if ($lvl < 2) {
+		    my @c = caller (2);
+		    if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
+			my $hints = $c[10];
+			(exists $hints->{autodie} && $hints->{autodie} or
+			 exists $hints->{"guard Fatal"} &&
+			!exists $hints->{"no Fatal"}) and
+			    $lvl++;
+			# Future releases of autodie will probably set $^H{autodie}
+			#  to "autodie @args", like "autodie :all" or "autodie open"
+			#  so we can/should check for "open" or "new"
+			}
 		    }
+		$lvl > 1 ? die $msg : warn $msg;
+		}
+	    else {	# called without args in void context
+		warn $msg;
 		}
-	    $lvl > 1 ? die $msg : warn $msg;
 	    }
 	return;
 	}
@@ -902,6 +928,13 @@ a TRUE value, then all fields will be quoted. This is typically easier
 to handle in external applications. (Poor creatures who aren't using
 Text::CSV_XS. :-)
 
+=item quote_space
+
+By default, a space in a field would trigger quotation. As no rule
+exists this to be forced in CSV, nor any for the opposite, the default
+is true for safety. You can exclude the space from this trigger by
+setting this option to 0.
+
 =item keep_meta_info
 
 By default, the parsing of input lines is as simple and fast as
@@ -972,6 +1005,7 @@ is equivalent to
      sep_char            => ',',
      eol                 => $\,
      always_quote        => 0,
+     quote_space         => 1,
      binary              => 0,
      keep_meta_info      => 0,
      allow_loose_quotes  => 0,
@@ -1550,6 +1584,11 @@ or the escape character, as that will invalidate all parsing rules.
 Using C<allow_whitespace> when either C<escape_char> or C<quote_char> is
 equal to SPACE or TAB is too ambiguous to allow.
 
+=item 1003 "INI - \r or \n in main attr not allowed"
+
+Using default C<eol> characters in either C<sep_char>, C<quote_char>, or
+C<escape_char> is not allowed.
+
 =item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 
 When C<eol> has been set to something specific, other than the default,
@@ -1667,7 +1706,7 @@ flags. See ChangeLog releases 0.25 and on.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2009 H.Merijn Brand for PROCURA B.V.
+Copyright (C) 2007-2010 H.Merijn Brand for PROCURA B.V.
 Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
 Portions Copyright (C) 1997 Alan Citterman. All rights reserved.
 
